import type { ErrorMessageMode } from '@/types/axios'
import type { LoginParams, LoginResult, UserInfo } from './types'
import { defHttp } from '@/utils/http/axios'

export function loginApi(params: LoginParams, mode: ErrorMessageMode = 'modal') {
  return defHttp.post<LoginResult>({
    url: '/iot-auth/oauth/token',
    params: {
      username: params.username,
      password: params.password,
      grant_type: 'password',
      scope: 'all',
      type: 'account',
    },
    headers: {
      'Tenant-Id': params.tenantId,
    },
  }, {
    errorMessageMode: mode,
    joinParamsToUrl: true,
    isTransformResponse: false,
  })
}

export function getUserInfo() {
  return defHttp.get<UserInfo>({
    url: '/iot-system/user/info',
  })
}

export interface __MenuItem {
  id: string
  parentId: string
  code: string
  name: string
  alias: string
  path: string
  source: string
  sort: number
  category: number
  action: number
  isOpen: 1 | 2
  children?: __MenuItem[]
  visible: BooleanFlag
}

export function getUserRouters() {
  return defHttp.get<__MenuItem[]>({
    url: '/iot-system/sysApp/routes?appCode=mes',
  })
}

export function getUserButtons() {
  return defHttp.get({
    url: '/iot-system/sys-app-menu/buttons?appCode=mes',
  })
}

export function doLogout() {
  return defHttp.get({
    url: '/iot-auth/oauth/logout',
  })
}

export function getLoginCaptcha() {
  return defHttp.get<{ image: string, key: string }>({
    url: '/baymax-auth/oauth/captcha',
  }, {
    isTransformResponse: false,
    withoutAuth: true,
  })
}

export function updateUserInfo(data: Partial<UserInfo>) {
  return defHttp.post({
    url: '/iot-system/user/update-info',
    data,
  })
}

export function updatePassword(params: { oldPassword: string, newPassword: string, newPassword1: string }) {
  return defHttp.post({
    url: '/iot-system/user/update-password',
    params,
  }, { joinParamsToUrl: true })
}
