import type { GetUserListParams, SystemUser } from './types'
import { defHttp } from '@/utils/http/axios'

export function getUserList(params: GetUserListParams) {
  return defHttp.get<PageResult<SystemUser>>({
    url: '/baymax-system/user/page',
    params,
  })
}

export function createUser(data: Partial<SystemUser>) {
  return defHttp.post({
    url: '/baymax-system/user/save',
    data,
  })
}

export function updateUser(data: Partial<SystemUser>) {
  return defHttp.post({
    url: '/baymax-system/user/update',
    data,
  })
}

export function deleteUser(ids: string[]) {
  return defHttp.post({
    url: `/baymax-system/user/remove?ids=${ids}`,
  })
}

export function resetPasswordByIds(ids: string[]) {
  return defHttp.post({
    url: `/baymax-system/user/reset-password?userIds=${ids}`,
  })
}

export function unlockUserByIds(ids: string[]) {
  return defHttp.post({
    url: `/baymax-system/user/unlock?userIds=${ids}`,
  })
}

export function exportUsers() {
  return defHttp.get({
    url: '/baymax-system/user/export-user',
    responseType: 'blob',
  })
}
