import type { RoleEnum } from '@/enums/roleEnum'
import type { RouteRecordRaw } from 'vue-router'
import { resetRouter, router } from '@/router'
import { useMultipleTabStore } from '@/store/modules/multipleTab'
import { usePermissionStore } from '@/store/modules/permission'
import { useUserStore } from '@/store/modules/user'
import { isArray } from '@/utils/is'
import { intersection } from 'lodash-es'
import { useTabs } from './useTabs'

// User permissions related operations
export function usePermission() {
  const userStore = useUserStore()
  const permissionStore = usePermissionStore()
  const { closeAll } = useTabs(router)

  /**
   * Reset and regain authority resource information
   * 重置和重新获得权限资源信息
   */
  async function resume() {
    const tabStore = useMultipleTabStore()
    tabStore.clearCacheTabs()
    resetRouter()
    const routes = await permissionStore.buildRoutesAction()
    routes.forEach((route) => {
      try {
        router.addRoute(route as unknown as RouteRecordRaw)
      }
      catch {}
    })
    permissionStore.setLastBuildMenuTime()
    closeAll()
  }

  /**
   * Determine whether there is permission
   */
  function hasPermission(value?: RoleEnum | RoleEnum[] | string | string[], def = true): boolean {
    // Visible by default
    if (!value)
      return def

    const allCodeList = permissionStore.getPermCodeList as string[]
    if (!isArray(value))
      return allCodeList.includes(value)

    return (intersection(value, allCodeList)).length > 0
  }

  /**
   * Change roles
   * @param roles
   */
  async function changeRole(roles: RoleEnum | RoleEnum[]): Promise<void> {
    if (!isArray(roles))
      roles = [roles]

    userStore.setRoleList(roles)
    await resume()
  }

  /**
   * refresh menu data
   */
  function refreshMenu() {
    return resume()
  }

  return { changeRole, hasPermission, refreshMenu }
}
