import type { Menu, MenuModule } from '@/router/types'
import { getAllParentPath, transformMenuModule } from '@/router/helper/menuHelper'
import { usePermissionStore } from '@/store/modules/permission'

const modules = import.meta.glob('./modules/**/*.ts', { eager: true })

const menuModules: MenuModule[] = []

Object.keys(modules).forEach((key) => {
  const mod = (modules as Recordable)[key].default || {}
  const modList = Array.isArray(mod) ? [...mod] : [mod]
  menuModules.push(...modList)
})

// ===========================
// ==========Helper===========
// ===========================

const staticMenus: Menu[] = []
;(() => {
  menuModules.sort((a, b) => {
    return (a.orderNo || 0) - (b.orderNo || 0)
  })

  for (const menu of menuModules)
    staticMenus.push(transformMenuModule(menu))
})()

function getAsyncMenus() {
  const permissionStore = usePermissionStore()
  // 递归过滤所有隐藏的菜单
  const menuFilter = (items) => {
    return items.filter((item) => {
      const show = !item.meta?.hideMenu && !item.hideMenu
      if (show && item.children)
        item.children = menuFilter(item.children)

      return show
    })
  }

  return menuFilter(permissionStore.getBackMenuList)
}

export async function getMenus(): Promise<Menu[]> {
  const menus = await getAsyncMenus()
  return menus
}

export async function getCurrentParentPath(currentPath: string) {
  const menus = await getAsyncMenus()
  const allParentPath = getAllParentPath(menus, currentPath)
  return allParentPath?.[0]
}

// Get the level 1 menu, delete children
export async function getShallowMenus(): Promise<Menu[]> {
  const menus = await getAsyncMenus()
  const shallowMenuList = menus.map(item => ({ ...item, children: undefined }))
  return shallowMenuList
}

// Get the children of the menu
export async function getChildrenMenus(parentPath: string) {
  const menus = await getMenus()
  const parent = menus.find(item => item.path === parentPath)
  if (!parent || !parent.children || !!parent?.meta?.hideChildrenInMenu)
    return [] as Menu[]

  return parent.children
}
