import type { ComputedRef, CSSProperties, Ref } from 'vue'

export type DynamicProps<T> = {
  [P in keyof T]: Ref<T[P]> | T[P] | ComputedRef<T[P]>
}

export type StyleValue = string | CSSProperties | Array<StyleValue>

/**
 * 任意类型的异步函数
 */
export type AnyPromiseFunction = (...arg: any[]) => PromiseLike<any>

/**
 * 任意类型的普通函数
 */
export type AnyNormalFunction = (...arg: any[]) => any

/**
 * 任意类型的函数
 */
export type AnyFunction = AnyNormalFunction | AnyPromiseFunction

export type Mutable<T> = { -readonly [P in keyof T]: T[P] }
