import type { SystemDictbiz } from '@/api/system/dictbiz/types'
import type { BasicColumn, FormSchema } from '@/components/Table'
import { getTenantId } from '@/utils/auth'
import { Tag } from 'ant-design-vue'
import { h } from 'vue'

export const columns: BasicColumn<SystemDictbiz>[] = [
  {
    title: '字典编号',
    dataIndex: 'code',
    width: 180,
    customRender: ({ record }) => {
      const { code } = record
      return h(Tag, { color: 'blue' }, () => code)
    },
  },
  {
    title: '字典名称',
    dataIndex: 'dictValue',
    width: 180,
  },
  {
    title: '字典键值',
    dataIndex: 'dictKey',
    width: 50,
  },
  {
    title: '封存',
    dataIndex: 'isSealed',
    width: 50,
    customRender: ({ record }) => {
      const { isSealed } = record
      const str = isSealed === 0 ? '否' : '是'
      return h(Tag, { color: 'blue' }, () => str)
    },
  },

]

export const searchFormSchema: FormSchema[] = [
  {
    label: '字典编号',
    field: 'code',
    component: 'Input',
    colProps: { span: 6 },
  },
  {
    label: '字典名称',
    field: 'dictValue',
    component: 'Input',
    colProps: { span: 6 },
  },
]

export function getFormSchema(): FormSchema[] {
  return [
    {
      field: 'tenantId',
      show: false,
      component: 'Input',
      defaultValue: getTenantId(),
    },
    {
      field: 'id',
      show: false,
      component: 'Input',
    },
    {
      field: 'isDeleted',
      show: false,
      component: 'Input',
      defaultValue: 0,
    },
    {
      field: 'parentId',
      show: false,
      component: 'Input',
      defaultValue: '0',
    },
    {
      label: '字典编号',
      field: 'code',
      required: true,
      component: 'Input',
      componentProps: {
        disabled: true,
      },
      colProps: {
        span: 24,
      },
    },
    {
      label: '字典名称',
      field: 'dictValue',
      required: true,
      component: 'Input',
    },
    {
      label: '上级字典',
      field: 'parentName',
      required: true,
      component: 'Input',
      componentProps: {
        disabled: true,
      },
    },
    {
      label: '字典键值',
      field: 'dictKey',
      required: true,
      component: 'Input',
    },
    {
      label: '字典排序',
      field: 'sort',
      required: true,
      component: 'InputNumber',
    },
    {
      label: '封存',
      field: 'isSealed',
      required: true,
      defaultValue: 0,
      component: 'Switch',
      componentProps: {
        checkedChildren: '是',
        unCheckedChildren: '否',
        checkedValue: 1,
        unCheckedValue: 0,
      },
    },
    {
      label: '字典备注',
      field: 'remark',
      required: false,
      component: 'Input',
    },
  ]
}
