import type { SystemPost } from '@/api/system/post/types'
import type { BasicColumn, FormSchema } from '@/components/Table'
import { getSystemDictionary } from '@/api/base/common'
import { getTenantId } from '@/utils/auth'
import { Tag } from 'ant-design-vue'
import { h } from 'vue'

export const columns: BasicColumn<SystemPost>[] = [
  {
    title: '所属租户',
    dataIndex: 'tenantName',
  },
  {
    title: '岗位类型',
    dataIndex: 'categoryName',
    customRender: ({ record }) => {
      const { categoryName } = record
      return h(Tag, { color: 'blue' }, () => categoryName)
    },
  },
  {
    title: '岗位编号',
    dataIndex: 'postCode',
  },
  {
    title: '岗位名称',
    dataIndex: 'postName',
  },

  {
    title: '岗位排序',
    dataIndex: 'sort',
  },

]

export const searchFormSchema: FormSchema[] = [
  {
    label: '岗位类型',
    field: 'category',
    component: 'ApiSelect',
    componentProps: {
      api: () => getSystemDictionary('post_category'),
    },
    colProps: { span: 6 },
  },
  {
    label: '岗位编号',
    field: 'postCode',
    component: 'Input',
    colProps: { span: 6 },
  },
  {
    label: '岗位名称',
    field: 'postName',
    component: 'Input',
    colProps: { span: 6 },
  },
]

export function getFormSchema(): FormSchema[] {
  return [
    {
      field: 'tenantId',
      show: false,
      component: 'Input',
      defaultValue: getTenantId(),
    },
    {
      field: 'id',
      show: false,
      component: 'Input',
    },
    //
    {
      label: '岗位类型',
      field: 'category',
      required: true,
      component: 'ApiSelect',
      componentProps: {
        api: async () => {
          const res = await getSystemDictionary('post_category')
          res.forEach((item) => {
            item.dictKey = Number(item.dictKey)
          })
          return res
        },
        valueField: 'dictKey',
        labelField: 'dictValue',
      },

    },
    {
      label: '岗位编号',
      field: 'postCode',
      required: true,
      component: 'Input',
    },
    {
      label: '岗位名称',
      field: 'postName',
      required: true,
      component: 'Input',
    },
    {
      label: '岗位排序',
      field: 'sort',
      required: true,
      component: 'InputNumber',
    },
    {
      label: '岗位描述',
      field: 'remark',
      required: false,
      component: 'InputTextArea',
      colProps: {
        span: 24,
      },
    },
  ]
}
