import { exportRegion } from '@/api/system/region'
import { useModal } from '@/components/Modal'
import { useMessage } from '@/hooks/web/useMessage'
import { noop } from '@/utils'
import { downloadByData } from '@/utils/file/download'
import { computed, ref } from 'vue'

export function useRegionActions() {
  function handleExport() {
    useMessage().createConfirm({
      title: '确定要导出行政区划数据吗？',
      iconType: 'warning',
      onOk() {
        exportRegion()
          .then((res) => {
            downloadByData(res, '行政区划.xlsx')
          })
          .catch(noop)
      },
    })
  }

  const [registerImportModal, { openModal: openImportModal }] = useModal()
  const isCovered = ref(0)
  const importTemplateUrl = '/baymax-system/region/export-template'
  const importUrl = computed(() => {
    return `/baymax-system/region/import-region?isCovered=${isCovered.value}`
  })

  return {
    handleExport,
    registerImportModal,
    openImportModal,
    importUrl,
    importTemplateUrl,
    isCovered,
  }
}
