import type { Region } from '@/api/system/region/types'
import { getSystemDictionary } from '@/api/base/common'
import { deleteRegion, updateRegion } from '@/api/system/region'
import { useForm } from '@/components/Form'
import { useMessage } from '@/hooks/web/useMessage'
import { noop } from '@/utils'

export function useRegionForm() {
  const [registerForm, methods] = useForm({
    schemas: [
      {
        field: 'parentCode',
        fields: [
          'id',
          'parentId',
          'code',
          'ancestors',
          'cityCode',
          'cityName',
          'districtCode',
          'districtName',
          'parentName',
          'provinceCode',
          'provinceName',
          'townCode',
          'townName',
          'villageCode',
          'villageName',
        ],
        label: '父区划编号',
        component: 'Input',
        render({ values }) {
          return values.parentCode
        },
      },
      {
        field: 'parentName',
        label: '父区划名称',
        component: 'Input',
        render({ values }) {
          return values.parentName
        },
      },
      {
        field: 'subCode',
        label: '区划编号',
        required: true,
        component: 'Input',
        componentProps({ formModel }) {
          return {
            addonBefore: formModel.parentCode,
          }
        },
      },
      {
        field: 'name',
        label: '区划名称',
        required: true,
        component: 'Input',
      },
      {
        field: 'regionLevel',
        label: '区划等级',
        required: true,
        component: 'ApiRadioGroup',
        componentProps: {
          api: () => getSystemDictionary('region'),
        },
      },
      {
        field: 'sort',
        label: '区划排序',
        required: true,
        component: 'InputNumber',
      },
      {
        field: 'remark',
        label: '区划备注',
        component: 'InputTextArea',
      },
    ],
    labelWidth: 80,
    baseColProps: {
      span: 24,
      style: {
        textAlign: 'left',
      },
    },
    actionColOptions: {
      span: 24,
      style: {
        textAlign: 'center',
      },
    },
    submitButtonOptions: {
      text: '保存',
    },
  })

  function handleSubmit(data: Region) {
    data = { ...data }
    const TOP_CODE = '00'
    data.parentCode = data.parentCode === TOP_CODE ? '' : data.parentCode
    data.code = data.parentCode + data.subCode
    updateRegion(data)
      .then(() => {
        useMessage().createMessage.success('保存成功')
      })
      .catch(noop)
  }

  function handleDelete(id: string) {
    deleteRegion(id)
      .then(() => {
        useMessage().createMessage.success('删除成功')
      })
      .catch(noop)
  }

  return {
    registerForm,
    ...methods,
    handleSubmit,
    handleDelete,
  }
}
