import type { Region } from '@/api/system/region/types'
import type { TreeItem } from '@/components/Tree'
import { lazyGetRegionList } from '@/api/system/region'
import { ref } from 'vue'

export function useRegionList() {
  const regionList = ref<Region[]>([])
  async function lazyRegionList(node: TreeItem) {
    return lazyGetRegionList(node.key.toString())
      .then((res) => {
        return res.map((item) => {
          return {
            ...item,
            isLeaf: !item.hasChildren,
          }
        })
      })
      .catch(() => [])
  }

  const TOP_KEY = '00'

  lazyRegionList({ key: TOP_KEY })
    .then(res => regionList.value = res)

  const selectedRegionKey = ref<string>()

  return {
    lazyRegionList,
    regionList,
    selectedRegionKey,
    setSelectedRegion(key?: string) {
      selectedRegionKey.value = key
    },
    reload() {
      lazyRegionList({ key: TOP_KEY })
    },
  }
}
