import type { BasicColumn, FormSchema } from '@/components/Table'
import { getRoleTree } from '@/api/system/role'
import { getAllTenants } from '@/api/system/tenant'
import { getTenantId } from '@/utils/auth'

export const columns: BasicColumn[] = [
  {
    title: '角色名称',
    dataIndex: 'roleName',
    width: 150,
  },
  {
    title: '所属租户',
    dataIndex: 'tenantName',
    width: 150,
  },
  {
    title: '角色别名',
    dataIndex: 'roleAlias',
    width: 150,
  },
  {
    title: '角色排序',
    dataIndex: 'sort',
    width: 120,
  },
]

export const searchFormSchema: FormSchema[] = [
  {
    label: '角色名称',
    field: 'roleName',
    component: 'Input',
    colProps: { span: 6 },
  },
  {
    label: '所属租户',
    field: 'tenantId',
    component: 'ApiSelect',
    componentProps: {
      api: getAllTenants,
      valueField: 'tenantId',
      labelField: 'tenantName',
    },
    colProps: { span: 6 },
  },
  {
    label: '角色别名',
    field: 'roleAlias',
    component: 'Input',
    colProps: { span: 6 },
  },
]

export const formSchema: FormSchema[] = [
  {
    field: 'tenantId',
    show: false,
    component: 'Input',
    defaultValue: getTenantId(),
  },
  {
    field: 'id',
    show: false,
    component: 'Input',
  },
  {
    label: '角色名称',
    field: 'roleName',
    required: true,
    component: 'Input',
  },
  {
    label: '角色别名',
    field: 'roleAlias',
    required: true,
    component: 'Input',
  },
  {
    label: '角色排序',
    field: 'sort',
    required: true,
    defaultValue: 0,
    component: 'InputNumber',
  },
  {
    label: '上级角色',
    field: 'parentId',
    component: 'ApiTreeSelect',
    componentProps: {
      api: () => getRoleTree({ tenantId: getTenantId() }),
      valueField: 'id',
      labelField: 'title',
    },
  },
]

export const menuScopeFormSchema: FormSchema[] = [
  {
    label: '角色编号',
    field: 'id',
    show: false,
    component: 'Input',
  },
  {
    label: '角色名称',
    field: 'name',
    dynamicDisabled: true,
    component: 'Input',
  },
  {
    label: '角色标识',
    field: 'code',
    dynamicDisabled: true,
    component: 'Input',
  },
  {
    label: '菜单权限',
    field: 'menuIds',
    slot: 'menuIds',
  },
]
