import type { BasicColumn, FormSchema } from '@/components/Table'
import { Tag } from 'ant-design-vue'
import { h } from 'vue'

export const columns: BasicColumn[] = [
  {
    title: '租户 ID',
    dataIndex: 'tenantId',
    width: 150,
  },
  {
    title: '联系人',
    dataIndex: 'linkman',
    width: 150,
  },
  {
    title: '联系电话',
    dataIndex: 'contactNumber',
    width: 150,
  },
  {
    title: '账号额度',
    dataIndex: 'accountNumber',
    width: 150,
    customRender({ value }) {
      return h(Tag, { color: value ? 'blue' : 'default' }, () => value)
    },
  },
  {
    title: '过期时间',
    dataIndex: 'expireTime',
    width: 150,
    customRender({ value }) {
      return h(Tag, { color: value ? 'default' : 'blue' }, () => value || '无限制')
    },
  },
]

export const searchFormSchema: FormSchema[] = [
  {
    label: '租户 ID',
    field: 'tenantId',
    component: 'Input',
    colProps: { span: 5 },
  },
  {
    label: '租户名称',
    field: 'tenantName',
    component: 'Input',
    colProps: { span: 5 },
  },
  {
    label: '联系人名称',
    field: 'linkman',
    component: 'Input',
    colProps: { span: 5 },
  },
]

export const formSchema: FormSchema[] = [
  {
    field: 'id',
    show: false,
    component: 'Input',
  },
  {
    label: '租户名称',
    field: 'tenantName',
    required: true,
    component: 'Input',
  },
  {
    label: '联系人名称',
    field: 'linkman',
    required: true,
    component: 'Input',
  },
  {
    label: '联系人电话',
    field: 'contactNumber',
    component: 'InputNumber',
    componentProps: {
      controls: false,
      precision: 0,
    },
  },
  {
    label: '联系人地址',
    field: 'address',
    component: 'InputTextArea',
  },
  {
    label: '绑定域名',
    field: 'domainUrl',
    component: 'Input',
  },
]

export const authSettingSchema: FormSchema[] = [
  {
    field: 'id',
    show: false,
    component: 'Input',
  },
  {
    label: '账号额度',
    field: 'accountNumber',
    component: 'InputNumber',
    componentProps: {
      controls: true,
      precision: 0,
    },
  },
  {
    label: '过期时间',
    field: 'expireTime',
    component: 'DatePicker',
    componentProps: {
      format: 'YYYY-MM-DD 00:00:00',
      valueFormat: 'YYYY-MM-DD 00:00:00',
      style: {
        width: '100%',
      },
    },
  },
]
